<?php
/**
 * Support for the Elementor Plugin
 * Plugin: https://elementor.com/
 *
 * @package PeachPay
 */

namespace Elementor;

if ( ! defined( 'PEACHPAY_ABSPATH' ) ) {
	exit;
}

/**
 * Elementor widget that inserts the PeachPay button onto any page.
 */
class PeachPay_Elementor_Widget extends Widget_Base {
	//phpcs:ignore
	public function __construct( $data = array(), $args = null ) {
		parent::__construct( $data, $args );
		wp_register_style(
			'enqueue_peachpay_css',
			peachpay_url( 'public/css/peachpay.css' ),
			array(),
			peachpay_file_version( 'public/css/peachpay.css' ),
		);
		wp_register_script(
			'enqueue_peachpay_js',
			peachpay_url( 'public/js/compatibility/peachpay-elementor.js' ),
			array( 'elementor-frontend' ),
			peachpay_file_version( 'public/js/compatibility/peachpay-elementor.js' ),
			true
		);
	}
//phpcs:ignore
	public function get_style_depends() {
		return array( 'enqueue_peachpay_css' );
	}
//phpcs:ignore
	public function get_script_depends() {
		return array( 'enqueue_peachpay_js' );
	}
//phpcs:ignore
	public function get_name() {
		return 'peachpay';
	}
//phpcs:ignore
	public function get_title() {
		return __( 'PeachPay', 'peachpay-for-woocommerce' );
	}
//phpcs:ignore
	public function get_icon() {
		return 'eicon-cart-solid';
	}

	/**
	 * Get widget categories.
	 *
	 * Retrieve the list of categories the PeachPay widget belongs to.
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return array( 'woocommerce-elements' );
	}

	/**
	 * Register PeachPay widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 */
	protected function register_controls() {
		$options = get_option( 'peachpay_button_options' );

		$this->start_controls_section(
			'content_section',
			array(
				'label' => __( 'Content', 'peachpay-for-woocommerce' ),
				'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
			)
		);

		$this->add_control(
			'text',
			array(
				'label'       => __( 'Button Text', 'peachpay-for-woocommerce' ),
				'type'        => \Elementor\Controls_Manager::TEXT,
				'placeholder' => 'Express Checkout',
			)
		);

		$this->add_control(
			'color',
			array(
				'label'   => __( 'Button Color', 'peachpay-for-woocommerce' ),
				'type'    => \Elementor\Controls_Manager::COLOR,
				'default' => $options['button_color'] ?? '#FF876C',
			)
		);

		$this->add_control(
			'text_color',
			array(
				'label'   => __( 'Text Color', 'peachpay-for-woocommerce' ),
				'type'    => \Elementor\Controls_Manager::COLOR,
				'default' => $options['button_text_color'] ?? '#FFFFFF',
			)
		);

		$this->add_control(
			'corners',
			array(
				'label'   => __( 'Rounded Corners', 'peachpay-for-woocommerce' ),
				'type'    => \Elementor\Controls_Manager::NUMBER,
				'min'     => 0,
				'max'     => 50,
				'default' => $options['button_border_radius'] ?? 5,
			)
		);

		$this->add_control(
			'icon',
			array(
				'label'   => __( 'Icon', 'peachpay-for-woocommerce' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'options' => array(
					'lock'          => esc_html__( 'Lock', 'peachpay-for-woocommerce' ),
					'baseball'      => esc_html__( 'Baseball', 'peachpay-for-woocommerce' ),
					'arrow'         => esc_html__( 'Arrow', 'peachpay-for-woocommerce' ),
					'mountain'      => esc_html__( 'Mountain', 'peachpay-for-woocommerce' ),
					'bag'           => esc_html__( 'Bag', 'peachpay-for-woocommerce' ),
					'shopping_cart' => esc_html__( 'Shopping Cart', 'peachpay-for-woocommerce' ),
					'none'          => esc_html__( 'None', 'peachpay-for-woocommerce' ),
				),
				'default' => $options['button_icon'] ?? 'none',
			)
		);

		$this->add_control(
			'fade',
			array(
				'label'     => __( 'Fade effect on hover', 'peachpay-for-woocommerce' ),
				'type'      => \Elementor\Controls_Manager::SWITCHER,
				'label_on'  => 'yes',
				'label_off' => 'no',
				'default'   => 'fade' === $options['button_effect'] ? 'yes' : 'no' ?? 'none',
			)
		);

		$this->add_control(
			'display_cards',
			array(
				'label'     => __( 'Display payment method icons below', 'peachpay-for-woocommerce' ),
				'type'      => \Elementor\Controls_Manager::SWITCHER,
				'label_on'  => 'yes',
				'label_off' => 'no',
				'default'   => $options['button_display_payment_method_icons'] ? 'yes' : 'no' ?? 'yes',
			)
		);

		$this->add_control(
			'default_fonts',
			array(
				'label'     => __( 'Font style match the theme font', 'peachpay-for-woocommerce' ),
				'type'      => \Elementor\Controls_Manager::SWITCHER,
				'label_on'  => 'yes',
				'label_off' => 'no',
				'default'   => 'no',
			)
		);

		$this->add_control(
			'alignment',
			array(
				'label'   => __( 'Alignment', 'peachpay-for-woocommerce' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'options' => array(
					'left'   => __( 'Left', 'peachpay-for-woocommerce' ),
					'right'  => __( 'Right', 'peachpay-for-woocommerce' ),
					'full'   => __( 'Full', 'peachpay-for-woocommerce' ),
					'center' => __( 'Center', 'peachpay-for-woocommerce' ),
				),
				'default' => $options['product_button_alignment'] ?? 'left',
			)
		);

		$this->add_control(
			'width',
			array(
				'label'   => __( 'Width', 'peachpay-for-woocommerce' ),
				'type'    => \Elementor\Controls_Manager::NUMBER,
				'min'     => 0,
				'max'     => 400,
				'step'    => 5,
				'default' => $options['button_width_product_page'] ?? 220,
			)
		);

		$this->end_controls_section();
	}

	/**
	 * Gets the icon URL based on the selected icon.
	 *
	 * @param string $icon Selected icon.
	 */
	private function get_icon_url( $icon ) {
		switch ( $icon ) {
			case 'lock':
				return peachpay_url( 'public/img/lock.svg' );
			case 'baseball':
				return peachpay_url( 'public/img/baseball-ball-solid.svg' );
			case 'arrow':
				return peachpay_url( 'public/img/chevron-circle-right-solid.svg' );
			case 'mountain':
				return peachpay_url( 'public/img/mountain-black.svg' );
			case 'bag':
				return peachpay_url( 'public/img/briefcase-solid.svg' );
			case 'shopping_cart':
				return peachpay_url( 'public/img/shopping-cart-solid.svg' );
			case 'none':
			default:
				return '';
		}
	}

	/**
	 * Render PeachPay widget output on the frontend.
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();
		if ( 'full' === $settings['alignment'] ) {
			$width = 'width: 100%;';
		} else {
			$width = 'width:' . $settings['width'] . 'px;';
		}
		$color       = '--pp-button-background-color:' . $settings['color'] . ';';
		$style       = $width . $color;
		$button_text = $settings['text'] ? $settings['text'] : 'Express Checkout';
		$icon_url    = $this->get_icon_url( $settings['icon'] );
		$alignment   = 'text-align:' . $settings['alignment'] . ' !important; justify-content: ' . $settings['alignment'] . ' !important;';
		?>
		<div id="pp-button-container" class="button-container pp-button-container " style="<?php echo esc_html( $settings['alignment'] ) !== 'full' ? esc_html( $alignment ) : ''; ?>">
			<div id="pp-stripe-payment-request-btn" style="width: 100%; margin: 5px 0; display: none;">
			</div>
			<button id="pp-button" class="pp-button pp-product-page <?php echo esc_html( $settings['default_fonts'] ) === 'yes' ? '' : 'pp-button-default-font'; ?> <?php echo esc_html( $settings['fade'] ) === 'yes' ? 'pp-effect-fade' : ''; ?> elementor-pp-button" type="button" style="display: block; <?php echo esc_html( $style ); ?> ?>; --pp-button-text-color:<?php echo esc_html( $settings['text_color'] ); ?>; border-radius: <?php echo esc_html( $settings['corners'] ); ?>px; font-size: 16px;">
				<object type="image/svg+xml" data="<?php echo esc_html( peachpay_url( 'public/img/spinner.svg' ) ); ?>" id="loading-spinner" class="pp-spinner hide" height="20" width="20">
				</object>
				<div id="pp-button-content">
					<span id="pp-button-text"><?php echo esc_html( $button_text ); ?></span>
					<img id="button-icon-regular" class="pp-btn-symbol <?php echo esc_html( $settings['icon'] ) === 'none' ? 'hide' : ''; ?>" src="<?php echo esc_html( $icon_url ); ?>">
				</div>
			</button>
			<div id="payment-methods-container" class="cc-company-logos <?php echo esc_html( $settings['display_cards'] ) === 'yes' ? '' : 'hide'; ?>">
				<img class="cc-logo" src="<?php echo esc_html( peachpay_url( 'public/img/marks/paypal.svg' ) ); ?>">
				<img class="cc-logo" src="<?php echo esc_html( peachpay_url( 'public/img/marks/affirm.svg' ) ); ?>">
				<img class="cc-logo" src="<?php echo esc_html( peachpay_url( 'public/img/marks/klarna.svg' ) ); ?>">
				<img class="cc-logo" src="<?php echo esc_html( peachpay_url( 'public/img/marks/afterpay.svg' ) ); ?>">
				<img class="cc-logo" src="<?php echo esc_html( peachpay_url( 'public/img/marks/visa.svg' ) ); ?>">
				<img class="cc-logo" src="<?php echo esc_html( peachpay_url( 'public/img/marks/amex.svg' ) ); ?>">
				<img class="cc-logo" src="<?php echo esc_html( peachpay_url( 'public/img/marks/mastercard.svg' ) ); ?>">
			</div>
		</div>
		<?php
	}
}

