const zeroDecimalCurrencies = new Set([
	'BIF',
	'CLP',
	'DJF',
	'GNF',
	'JPY',
	'KMF',
	'KRW',
	'MGA',
	'PYG',
	'RWF',
	'UGX',
	'VND',
	'VUV',
	'XAF',
	'XOF',
	'XPF'
]);

// Hide and show the table for all rates.
document.querySelector('#pp-show-all-rates').addEventListener('click', () => {
	const rates = document.querySelector('#pp-all-currency-table');
	if( rates.classList.contains('hide') ) {
		rates.classList.remove('hide');
		event.target.value='Hide rates';
	} else {
		rates.classList.add('hide');
		event.target.value = 'Show rates';
	}

});

// Event listener to show the custom interval setting.
document.querySelector('#enable_peachpay_currency_custom_intervals').addEventListener('change', hideCustomIntervals);

// Remove row event.
document.querySelectorAll('.pp-removeButton').forEach( (element) => {
	element.addEventListener('click', removeRowAndUpdate, {element});
});

// Event for listening if every currency is checked hide other table and settings since it overides all that.
const everyCurrency = document.querySelector('#peachpay_enable_every_currency');
everyCurrency.addEventListener('change', allCurrencyHide);
if(everyCurrency.checked) {
	document.querySelector('.currency_table_section').classList.add('hide');
	document.querySelector('.currency_update_frequency').classList.add('hide');
	document.querySelector('.custom_frequencies').classList.add('hide');
} 

// Event listener for zero decimal currencies setup.
document.querySelectorAll('.name').forEach( (element) => {
	if (zeroDecimalCurrencies.has(element.value)){
		element.parentElement.parentElement.querySelector('.decimals').value = 0;
		element.parentElement.parentElement.querySelector('.decimals').readOnly = true;
	}
	element.addEventListener('change', checkTwoDecimal, {element});
});

// Change the list of supported payment methods through JS.
document.querySelectorAll('#peachpay_new_currency_code').forEach( (element) =>{
	element.addEventListener('change', updateSupported, );
});

// Change rate to read only if auto-update enabled.
document.querySelectorAll('.auto_update').forEach( (element) => {
	if (element.checked){
		element.parentElement.parentElement.querySelector('.rate').readOnly = true;
	}
	element.addEventListener('change', checkAutoUpdate, {element});
});

// Add a new currency instead of using a number use an update flag now so number can't get corrupted or lost.
const $updateCurrency = document.querySelector("#updateCurrency");
if ($updateCurrency) {
	$updateCurrency.addEventListener('click', () => {
		let x = document.querySelector("#hiddenAddFlag");
		x.disabled=false;
		document.querySelector("#submit").click();
	});
}

function removeRowAndUpdate() {
	const row = event.target.closest('tr');
	const children = row.querySelectorAll('input,select');
	children.forEach(element => {
		element.disabled = true;

	});
	document.querySelector("#submit").click();
}

function checkTwoDecimal() {
	if (zeroDecimalCurrencies.has(event.target.value)){
		event.target.parentElement.parentElement.querySelector('.decimals').value = 0;
		event.target.parentElement.parentElement.querySelector('.decimals').readOnly = true;
		return;
	}
	event.target.parentElement.parentElement.querySelector('.decimals').readOnly = false;
}

function checkAutoUpdate() {
	if (event.target.checked){
		event.target.parentElement.parentElement.querySelector('.rate').readOnly = true;
		return;
	}
	event.target.parentElement.parentElement.querySelector('.rate').readOnly = false;
}

function updateSupported(event) {
	let country = event.target.value;
	const active_providers = pp_currency_data.active_providers;
	const supports = pp_currency_data.method_supports[country];

	const difference = active_providers.filter( provider =>  !supports.includes(provider));
	
	if(difference.length !== 0){
		const tooltip = event.target.closest('td').querySelector('.pp-tooltipHidden');
		const warning = event.target.closest('td').querySelector('.pp-method-warning');
		tooltip.classList.remove('hide');
		warning.classList.remove('hide');
		tooltip.innerHTML = `Not supported by the following providers: ${difference.join(', ')}`
	} else {
		const tooltip = event.target.closest('td').querySelector('.pp-tooltipHidden');
		const warning = event.target.closest('td').querySelector('.pp-method-warning');
		tooltip.classList.add('hide');
		warning.classList.add('hide');
	}

}

function hideCustomIntervals() {
	if(event.target.checked) {
		document.querySelectorAll('.custom_interval_selector').forEach((element) =>{
			element.classList.remove('hide');
		});
	} else {
		document.querySelectorAll('.custom_interval_selector').forEach((element) =>{
			element.classList.add('hide');
		});
	}
}

function allCurrencyHide( event ) {
	if (event.target.checked) {
		document.querySelector('.currency_table_section').classList.add('hide');
		document.querySelector('.currency_update_frequency').classList.add('hide');
		document.querySelector('#pp-show-all-rates').classList.remove('hide');
		document.querySelector('.custom_frequencies').classList.add('hide');
	} else {
		document.querySelector('.currency_table_section').classList.remove('hide');
		document.querySelector('.currency_update_frequency').classList.remove('hide');
		document.querySelector('#pp-show-all-rates').classList.add('hide');
		document.querySelector('.custom_frequencies').classList.remove('hide');
	}
}

document.querySelector('#submit').addEventListener('click', (event) => {
	let countries = document.querySelectorAll('.countries').forEach((element)=>{
		const td = element.closest('td');
		const selected = td.querySelector('.currencyCountries').selectedOptions;
		for(let i = 0; i < selected.length; i++) {
			if(!element.value.includes(selected[i].value)) {
				element.value += ',' + selected[i].value;
			}
		}
	})
});
