<?php
/**
 * File to hold all settings related to Peachpay Currency Switcher
 *
 * @package PeachPay
 */

if ( ! defined( 'PEACHPAY_ABSPATH' ) ) {
	exit;
}

require_once PEACHPAY_ABSPATH . '/core/modules/currency-switcher/util/peachpay-currency-arrays.php';

/**
 * New settings for our built in peachpay currency switcher allows admins to view and set settings for our currency switcher itself.
 */
function peachpay_settings_currency_switch() {
	add_settings_section(
		'peachpay_section_currency_switch',
		__( 'Currency switcher', 'peachpay-for-woocommerce' ),
		'peachpay_feedback_cb',
		'peachpay',
		'peachpay_section_currency',
	);

	add_settings_field(
		'peachpay_field_enabled_currency_switch',
		__( 'Enabled', 'peachpay-for-woocommerce' ),
		'peachpay_enabled_currency_cb',
		'peachpay',
		'peachpay_section_currency_switch',
	);

	add_settings_field(
		'peachpay_field_currency_change_currency_every_country',
		__( 'Support every currency', 'peachpay-for-woocommerce' ),
		'peachpay_every_currency_cb',
		'peachpay',
		'peachpay_section_currency_switch'
	);

	add_settings_field(
		'peachpay_field_currency_update_time',
		__( 'Frequency of auto-update', 'peachpay' ),
		'peachpay_update_frequency_cb',
		'peachpay',
		'peachpay_section_currency_switch',
		array( 'class' => 'currency_update_frequency' )
	);

	add_settings_field(
		'peachpay_field_custom_intervals',
		__( 'Custom intervals', 'peachpay-for-woocommerce' ),
		'peachpay_currency_custom_interval_cb',
		'peachpay',
		'peachpay_section_currency_switch',
		array( 'class' => 'custom_frequencies' )
	);

	add_settings_field(
		'peachpay_field_currencies',
		__( 'Currencies', 'peachpay-for-woocommerce' ),
		'peachpay_currencies_cb',
		'peachpay',
		'peachpay_section_currency_switch',
		array( 'class' => 'currency_table_section' )
	);

}

/**
 * If currency switch is enabled or not.
 */
function peachpay_enabled_currency_cb() {
	?>
	<input type="checkbox"
	name="peachpay_currency_options[enabled]"
	id= "enable_peachpay_currency_switch"
	value = 1
	<?php checked( 1, peachpay_get_settings_option( 'peachpay_currency_options', 'enabled' ), true ); ?>
	>
	<label for="enable_peachpay_currency_switch">
		<?php esc_html_e( 'Enable the currency switcher', 'peachpay-for-woocommerce' ); ?>
	</label>
	<?php
}

/**
 * If a merchant wants to support every currency we support let them :)
 */
function peachpay_every_currency_cb() {
	$all_currencies = peachpay_get_settings_option( 'peachpay_currency_options', 'every_currency_table', array() );
	$base           = get_option( 'woocommerce_currency' );
	?>
	<input type="checkbox"
	name="peachpay_currency_options[every_currency]"
	id= "peachpay_enable_every_currency"
	value = 1
	<?php checked( 1, peachpay_get_settings_option( 'peachpay_currency_options', 'every_currency' ), true ); ?>
	>
	<label for="enable_every_currency">
		<?php esc_html_e( 'If checked, the table below will be disabled, and instead your store will support all 135 currencies that PeachPay supports', 'peachpay-for-woocommerce' ); ?>
		<p class='description'>
		<?php esc_html_e( 'PeachPay will select the best currency for a shopper based on their location. The initial rates will populate upon saving the settings.', 'peachpay-for-woocommerce' ); ?>
		</p>
	</label>
	<input type="button" id="pp-show-all-rates"
		value="<?php echo esc_html_e( 'Show rates', 'peachpay-for-woocommerce' ); ?>"
		class="<?php echo peachpay_get_settings_option( 'peachpay_currency_options', 'every_currency' ) ? '' : esc_html( 'hide' ); ?>"
		style="margin-top: 1rem;"
	>
	<table class="hide" id="pp-all-currency-table">
		<?php
		$i = 0;
		foreach ( PEACHPAY_SUPPORTED_CURRENCIES as $code => $name ) {
			?>
		<tr>
			<td class="pp-all-currency-td">
				<input type='hidden' name='peachpay_currency_options[every_currency_table][<?php echo esc_html( $code ); ?>][name]'
				value=<?php echo esc_html( $code ); ?> readonly>
				</input>
				<input type='text' class="pp-all-currency-show" readonly value="<?php echo esc_html( PEACHPAY_SUPPORTED_CURRENCIES[ $code ] ); ?>" >
				<input type='text' name='peachpay_currency_options[every_currency_table][<?php echo esc_html( $code ); ?>][rate]' class="pp-all-currency-show"
				value=<?php echo array_key_exists( $code, $all_currencies ) ? esc_html( $all_currencies[ $code ]['rate'] ) : 1; ?> readonly>
				<input type='hidden' name='peachpay_currency_options[every_currency_table][<?php echo esc_html( $code ); ?>][decimals]'
				value=<?php echo peachpay_is_zero_decimal_currency( $code ) ? 0 : 2; ?>>
				<input type='hidden' name='peachpay_currency_options[every_currency_table][<?php echo esc_html( $code ); ?>][round]'
				value="up">
				<input type='hidden' name='peachpay_currency_options[every_currency_table][<?php echo esc_html( $code ); ?>][auto_update]'
				value="1"
				<?php echo $code === $base ? esc_html( 'disabled' ) : ''; ?>
				>
			</td>
		</tr>
		<?php $i++; } ?>
	</table>
	<?php
}

/**
 * Callback for selecting currencies and conversion rates for peachpay
 */
function peachpay_currencies_cb() {
	$base_currency = get_option( 'woocommerce_currency' );

	$active_currencies = peachpay_get_settings_option( 'peachpay_currency_options', 'selected_currencies', array() );

	$types = array(
		'none'     => __( 'No custom interval', 'peachpay-for-woocommerce' ),
		'15minute' => __( 'Update every 15 minutes', 'peachpay-for-woocommerce' ),
		'30minute' => __( 'Update every 30 minutes', 'peachpay-for-woocommerce' ),
		'hourly'   => __( 'Update every hour', 'peachpay-for-woocommerce' ),
		'6hour'    => __( 'Update every 6 hours', 'peachpay-for-woocommerce' ),
		'12hour'   => __( 'Update every 12 hours', 'peachpay-for-woocommerce' ),
		'daily'    => __( 'Update every day', 'peachpay-for-woocommerce' ),
		'2day'     => __( 'Update every 2 days', 'peachpay-for-woocommerce' ),
		'weekly'   => __( 'Update once a week', 'peachpay-for-woocommerce' ),
		'biweekly' => __( 'Update every 2 weeks', 'peachpay-for-woocommerce' ),
		'monthly'  => __( 'Update every month', 'peachpay-for-woocommerce' ),
	);

	$active_payment_methods = peachpay_get_active_payment_methods();

	$round_values = array(
		'up',
		'down',
		'nearest',
		'none',
	);

	?>
	<div id='pp-currency-table-div'>
	<table id = "pp-active-currencies" >
	<tr class = table-header-footer >
		<td> </td>
		<th class= "pp-tooltip"><?php esc_html_e( ' Currency', 'peachpay-for-woocommerce' ); ?>
			<span>
				<img  class='pp-tooltip-qm' src=<?php echo esc_url( peachpay_url( '/core/modules/currency-switcher/admin/assets/Property_1help_isFilledTrue.svg' ) ); ?> >
			</span>
			<span class ="pp-tooltipHidden"> <?php esc_html_e( 'Currency converted to from base', 'peachpay-for-woocommerce' ); ?> </span>
		</th>

		<th class="pp-tooltip"><?php esc_html_e( ' Auto update ', 'peachpay-for-woocommerce' ); ?>
			<span>
				<img class='pp-tooltip-qm' src=<?php echo esc_url( peachpay_url( '/core/modules/currency-switcher/admin/assets/Property_1help_isFilledTrue.svg' ) ); ?> >
			</span>
			<span class ="pp-tooltipHidden"> <?php esc_html_e( 'Update the rate automatically?', 'peachpay-for-woocommerce' ); ?></span>
		</th>

		<th class="pp-tooltip"><?php esc_html_e( ' Conversion rate ', 'peachpay-for-woocommerce' ); ?>
			<span>
				<img class='pp-tooltip-qm' src=<?php echo esc_url( peachpay_url( '/core/modules/currency-switcher/admin/assets/Property_1help_isFilledTrue.svg' ) ); ?> >
			</span>
			<span class ="pp-tooltipHidden"> <?php esc_html_e( 'Rate at which the currency will be exchanged', 'peachpay-for-woocommerce' ); ?></span>
		</th>

		<th class="pp-tooltip"><?php esc_html_e( ' Decimals ', 'peachpay-for-woocommerce' ); ?>
		<span>
			<img class='pp-tooltip-qm' src=<?php echo esc_url( peachpay_url( '/core/modules/currency-switcher/admin/assets/Property_1help_isFilledTrue.svg' ) ); ?> >
		</span>
		<span class ="pp-tooltipHidden"><?php esc_html_e( 'Number of decimals the currency will support', 'peachpay-for-woocommerce' ); ?></span></th>

		<th class="pp-tooltip"><?php esc_html_e( 'Rounding', 'peachpay-for-woocommerce' ); ?>
		<span>
			<img class='pp-tooltip-qm' src=<?php echo esc_url( peachpay_url( '/core/modules/currency-switcher/admin/assets/Property_1help_isFilledTrue.svg' ) ); ?> >
		</span>
		<span class ="pp-tooltipHidden"><?php esc_html_e( 'How to round when the there are more decimals than the currency supports', 'peachpay-for-woocommerce' ); ?> </span></th>

		<th class="pp-tooltip custom_interval_selector <?php echo peachpay_get_settings_option( 'peachpay_currency_options', 'custom_intervals' ) ? '' : esc_html( 'hide' ); ?>"><?php esc_html_e( 'Custom interval', 'peachpay-for-woocommerce' ); ?>
		<span>
			<img class='pp-tooltip-qm' src=<?php echo esc_url( peachpay_url( '/core/modules/currency-switcher/admin/assets/Property_1help_isFilledTrue.svg' ) ); ?> >
		</span>
		<span class ="pp-tooltipHidden"><?php esc_html_e( 'Set this to "No custom interval" to follow the default interval schedule', 'peachpay-for-woocommerce' ); ?> </span></th>

		<th class="pp-tooltip"> <?php esc_html_e( 'Countries restricted to', 'peachpay-for-woocommerce' ); ?>
		<span>
			<img class='pp-tooltip-qm' src=<?php echo esc_url( peachpay_url( '/core/modules/currency-switcher/admin/assets/Property_1help_isFilledTrue.svg' ) ); ?> >
		</span>
		<span class ="pp-tooltipHidden"><?php esc_html_e( 'If a currency has countries associated with it, the currency will only be available in those countries.', 'peachpay-for-woocommerce' ); ?></span></th>
	</tr>

	<tr class="pp-base-currency">
		<td> <?php esc_html_e( 'Base currency', 'peachpay-for-woocommerce' ); ?></td>
		<td>
		<div>
		<?php echo esc_html( PEACHPAY_SUPPORTED_CURRENCIES[ $base_currency ] ); ?>
		</div>
		<?php
		$not_supported = array_diff( $active_payment_methods, PEACHPAY_CURRENCIES_METHOD_ARRAY[ $base_currency ] );
		?>
			<div class="pp-method-warning pp-tooltip <?php echo empty( $not_supported ) ? esc_html( 'hide' ) : ''; ?>">
			<img style='vertical-align:-2px' width=15px height=15px src =
			<?php
			echo esc_url( peachpay_url( 'core/modules/currency-switcher/admin/assets/warning-sign.svg' ) );
			?>
			>
			<span class="pp-tooltipHidden pp-currency-warning-table">
			<?php
			esc_html_e( 'Not supported by the following providers: ' );
			echo esc_html( implode( ' ', $not_supported ) );
			?>
			</span>
			</div>

		</td>
		<td> N/A </td>
		<td>1</td>
		<td><?php echo esc_html( get_option( 'woocommerce_price_num_decimals' ) ); ?></td>
		<td>N/A</td>
		<td class="custom_interval_selector <?php echo peachpay_get_settings_option( 'peachpay_currency_options', 'custom_intervals' ) ? '' : esc_html( 'hide' ); ?>"> N/A</td>
		<td>
		<select class="chosen-select" data-placeholder="Allowed in all non-restrcited countries" multiple id="pp_countries_base">
			<?php
			foreach ( ISO_TO_COUNTRY as $iso => $country ) {
				?>
					<option value =
					<?php
					echo esc_html( $iso );
					?>
					<?php
					$selected = peachpay_get_settings_option( 'peachpay_currency_options', 'selected_currencies', null );
					$selected = explode( ',', $selected['base']['countries'], 100000 );
					echo ( in_array( $iso, $selected, true ) ? esc_html( 'selected' ) : '' );
					?>
					>
					<?php echo esc_html( $country ); ?>
					</option>
					<?php
			}
			?>
			<script>
				window.addEventListener('load', () => {
					document.querySelector('#submit').addEventListener('click', (event) => {
					let select = document.querySelector('#pp_countries_base');
					let hiddenInput = document.querySelector( '#hiddenCountriesBase');
					let chosen = select.selectedOptions;
					for(let i = 0; i < chosen.length; i++) {
						if(!hiddenInput.value.includes(chosen[i].value)) {
							hiddenInput.value += ',' + chosen[i].value;
						}
					}
				});
			});
			</script>
		</td>
	</tr>
	<?php
	$i = 0;
	foreach ( $active_currencies as $key => $currency ) {
		if ( 'base' === $key ) {
			continue;
		}
		?>
		<tr class="currencyRow" >
		<td style="vertical-align:5px;">
			<input type = "button" value="Remove" class = "pp-removeButton">
		</td>

		<td class="pp-currency-name">
			<select
			id="peachpay_new_currency_code"
			name="peachpay_currency_options[selected_currencies][<?php echo esc_html( $i ); ?>][name]"
			value =
			<?php
				echo array_key_exists( 'name', $currency ) ? esc_html( $currency['name'] ) : esc_html( $base_currency );
			?>
			class = 'name'
			>
			<?php foreach ( PEACHPAY_SUPPORTED_CURRENCIES as $code => $name ) { ?>
				<option
					value="<?php echo esc_attr( $code ); ?>"
					<?php
					if ( array_key_exists( 'name', $currency ) ) {
						echo ( ( $currency['name'] === $code ) ? 'selected' : ' ' );
					}
					?>
				>
					<?php echo esc_html( $name ); ?>
				</option>
				<?php
			}
			?>
		</select>
		<?php
		$not_supported = array_diff( $active_payment_methods, PEACHPAY_CURRENCIES_METHOD_ARRAY[ $currency['name'] ] );
		?>
			<div class="pp-method-warning pp-tooltip <?php echo empty( $not_supported ) ? esc_html( 'hide' ) : ''; ?>">
			<img style='vertical-align:-2px' width=15px height=15px src =
			<?php
			echo esc_url( peachpay_url( 'core/modules/currency-switcher/admin/assets/warning-sign.svg' ) );
			?>
			>
			<span class="pp-tooltipHidden pp-currency-warning-table">
			<?php
			esc_html_e( 'Not supported by the following providers: ' );
			echo esc_html( implode( ' ', $not_supported ) );
			?>
			</span>
			</div>
		</td>

		<td>
		<input
			id = "peachpay_new_currency_auto_update"
			name = "peachpay_currency_options[selected_currencies][<?php echo esc_html( $i ); ?>][auto_update]"
			value ="1"
			<?php
			if ( array_key_exists( 'auto_update', $currency ) ) {
				echo( checked( '1', $currency['auto_update'] ) );
			}
			?>
			type="checkbox"
			class="auto_update">
		</td>

		<td>
			<input
			id = "peachpay_new_currency_rate"
			name = "peachpay_currency_options[selected_currencies][<?php echo esc_html( $i ); ?>][rate]"
			value =
			<?php
			if ( array_key_exists( 'rate', $currency ) ) {
				echo( esc_html( $currency['rate'] ) );
			} else {
				echo( 1 );
			}
			?>
			type="text"
			class="rate"
			<?php
			if ( array_key_exists( 'auto_update', $currency ) ) {
				echo( 'readonly' );
			}
			?>
			>
		</input>
		</td>

		<td>
			<input
			id = "peachpay_new_currency_decimals"
			name = "peachpay_currency_options[selected_currencies][<?php echo esc_html( $i ); ?>][decimals]"
			value = <?php echo esc_html( $currency['decimals'] ); ?>
			type="number"
			min=0
			max=2
			class="decimals">
		</td>

		<td>
			<select
			id = "peachpay_convert_rounding"
			name = "peachpay_currency_options[selected_currencies][<?php echo esc_html( $i ); ?>][round]"
			class = "round"
			>
			<?php
			foreach ( $round_values as $round ) {
				?>
				<option
					value= <?php echo esc_html( $round ); ?>
					<?php array_key_exists( 'round', $currency ) && $currency['round'] === $round ? esc_html( 'selected' ) : ''; ?>
					>
					<?php echo esc_html( $round ); ?>
				</option>
			<?php } ?>
			</select>
		</td>

		<td class="custom_interval_selector <?php echo peachpay_get_settings_option( 'peachpay_currency_options', 'custom_intervals' ) ? '' : esc_html( 'hide' ); ?>">
			<select
			name="peachpay_currency_options[selected_currencies][<?php echo esc_html( $i ); ?>][custom_interval]"
			>
				<?php
				$interval = array_key_exists( 'custom_interval', $currency ) ? $currency['custom_interval'] : 'none';
				foreach ( $types as $key => $type ) {
					?>
					<option value="<?php echo esc_html( $key ); ?>" <?php echo $interval === $key ? esc_html( 'selected' ) : ''; ?> > <?php echo esc_html( $type ); ?> </option>
					<?php
				}
				?>
			</select>
		</td>
		<td>
		<select class="chosen-select currencyCountries" data-placeholder="Allowed everywhere" multiple id="pp_countries<?php echo esc_html( $currency['name'] ); ?>">
			<?php
			foreach ( ISO_TO_COUNTRY as $iso => $country ) {
				?>
				<option value =
				<?php
				echo esc_html( $iso );
				?>
				<?php
				$selected = explode( ',', $currency['countries'], 100000 );
				echo ( in_array( $iso, $selected, true ) ? esc_html( 'selected' ) : '' );
				?>
				>
				<?php echo esc_html( $country ); ?>
				</option>
				<?php
			}

			?>
		</select>


		<input type='text' name="peachpay_currency_options[selected_currencies][<?php echo esc_html( $i ); ?>][countries]" id="hiddenCountries<?php echo esc_html( $currency['name'] ); ?>" class='countries' hidden>
		</td>
	</tr>
	<tr>
		<?php
		$i++;
	}
	?>
	<tr>
	<td>
	<input
	type = "hidden"
	name = "peachpay_currency_options[selected_currencies][base][name]"
	value = <?php echo esc_html( $base_currency ); ?>
	>
	</input>
	</td>
<td>
	<input
		type = "hidden"
		name = "peachpay_currency_options[selected_currencies][base][rate]"
		value =1
	>
	</input>
</td>
<td>
	<input
		type = "hidden"
		name = "peachpay_currency_options[selected_currencies][base][auto_update]"
		value = "1"
	>
	</input>
</td>
<td>
	<input
		type = "hidden"
		name = "peachpay_currency_options[selected_currencies][base][round]"
		value = "disabled"
	>
	</input>
</td>

<td>
	<input
		type = "hidden"
		name = "peachpay_currency_options[selected_currencies][base][decimals]"
		value = <?php echo esc_html( get_option( 'woocommerce_price_num_decimals' ) ); ?>
	>
	</input>
</td>

	<td>
		<input
			type = "hidden"
			name = "peachpay_currency_options[selected_currencies][base][decimals]"
			value = <?php echo esc_html( get_option( 'woocommerce_price_num_decimals' ) ); ?>
		>
		</input>
	</td>

	<td>
		<input
			type = "hidden"
			name = "peachpay_currency_options[selected_currencies][base][countries]"
			id ="hiddenCountriesBase"
			value =""
		>
		</input>
	</td>

	<td>
	<input
			type = "hidden"
			name = "peachpay_currency_options[new_flag]"
			id = "hiddenAddFlag";
			value =1
			disabled
		>
		</input>
	</td>

<tr>
<td>
	<input type="button" value ="Add new currency" id = "updateCurrency">
	<script>


	</script>
</td>
</tr>
</table>
</div>

	<script>
			jQuery(".chosen-select").chosen({
				no_results_text: "No matching country found"
			})

	</script>

	<?php
	echo ( ' </tr> </table>' );
}

/**
 * Callback for rendering currency auto update time.
 */
function peachpay_update_frequency_cb() {
	$types = array(
		'15minute' => 'Update every 15 minutes',
		'30minute' => 'Update every 30 minutes',
		'hourly'   => 'Update every hour',
		'6hour'    => 'Update every 6 hours',
		'12hour'   => 'Update every 12 hours',
		'daily'    => 'Update every day',
		'2day'     => 'Update every 2 days',
		'weekly'   => 'Update once a week',
		'biweekly' => 'Update every 2 weeks',
		'monthly'  => 'Update every month',
	);
	?>
	<select
	id = "peachpay_convert_type"
	name = "peachpay_currency_options[update_frequency]"
	class="currencyType">
	<?php
	foreach ( $types as $type => $type_value ) {
		?>
		<option
			value="<?php echo esc_attr( $type ); ?>"
			<?php
				echo ( peachpay_get_settings_option( 'peachpay_currency_options', 'update_frequency' ) === $type ? 'selected' : ' ' );
			?>
			>
			<?php echo esc_html( $type_value ); ?>
		</option>
	<?php } ?>
	</select>
	<?php
}

add_action( 'admin_enqueue_scripts', 'peachpay_enqueue_currency_admin_dropdown_scripts' );

/**
 * Enque scripts so our dropdown displays
 *
 * @param string $hook the top level page.
 */
function peachpay_enqueue_currency_admin_dropdown_scripts( $hook ) {
	if ( 'toplevel_page_peachpay' !== $hook ) {
		return;
	}
	wp_enqueue_script( 'pp_dropdown_code', 'https://cdn.rawgit.com/harvesthq/chosen/gh-pages/chosen.jquery.min.js', array(), '1.8.7', false );
	wp_enqueue_style( 'pp_dropdown_style', 'https://cdn.rawgit.com/harvesthq/chosen/gh-pages/chosen.min.css', array(), '1.8.7', 'all' );
}

add_action( 'admin_enqueue_scripts', 'peachpay_enqueue_currency_admin_scripts' );
/**
 * Enque our script that allows currency addition and removal
 *
 * @param string $hook the top level page.
 */
function peachpay_enqueue_currency_admin_scripts( $hook ) {
	if ( 'toplevel_page_peachpay' !== $hook ) {
		return;
	}
	wp_enqueue_script(
		'pp_currency',
		peachpay_url( 'core/modules/currency-switcher/admin/js/remove-row.js' ),
		array(),
		peachpay_file_version( 'core/modules/currency-switcher/admin/js/remove-row.js' ),
		true
	);

	wp_localize_script(
		'pp_currency',
		'pp_currency_data',
		array(
			'method_supports'  => PEACHPAY_CURRENCIES_METHOD_ARRAY,
			'active_providers' => peachpay_get_active_payment_methods(),
		)
	);
}

/**
 * Get active payment methods.
 */
function peachpay_get_active_payment_methods() {
	$active   = array();
	$stripe   = peachpay_get_settings_option( 'peachpay_payment_options', 'enable_stripe' );
	$klarna   = peachpay_get_settings_option( 'peachpay_payment_options', 'klarna_payments' );
	$afterpay = peachpay_get_settings_option( 'peachpay_payment_options', 'afterpay_clearpay_payments' );
	$paypal   = peachpay_get_settings_option( 'peachpay_payment_options', 'paypal' );

	$stripe ? array_push( $active, 'Stripe' ) : '';
	$klarna ? array_push( $active, 'Klarna' ) : '';
	$afterpay ? array_push( $active, 'Afterpay/Clearpay' ) : '';
	$paypal ? array_push( $active, 'PayPal' ) : '';

	return $active;
}

/**
 * Allow merchants to always have paypal in usd even if paypal doesn't support the selected currency.
 */
function peachpay_field_auto_convert_paypal() {
	$paypal_supported_currencies = array(
		'AUD',
		'CAD',
		'CZK',
		'DKK',
		'EUR',
		'HKD',
		'HUF',
		'ILS',
		'JPY',
		'MXN',
		'TWD',
		'NZD',
		'NOK',
		'PHP',
		'PLN',
		'GBP',
		'RUB',
		'SGD',
		'SEK',
		'CHF',
		'TBH',
		'USD',
	);
	$currencies                  = peachpay_get_settings_option( 'peachpay_currency_options', 'selected_currencies', array() );
	foreach ( $currencies as $key => $currency ) {
		if ( array_search( $currency['name'], $paypal_supported_currencies, true ) === false ) {
			unset( $currencies[ $key ] );
		}
	}
	if ( 1 > count( $currencies ) ) {
		return;
	}
	?>
	<h3>
		<?php esc_html_e( 'Default currency', 'peachpay_for_woocommerce' ); ?>
	<h3>
	<select
		id="peachpay_paypal_auto_usd"
		name="peachpay_payment_options[paypal_auto_convert]"
		value="none"
	>
	<option value='none'> None </option>
	<?php
	foreach ( $currencies as $currency ) {
		if ( in_array( $currency['name'], $paypal_supported_currencies, true ) ) {
			// update later to name of currency.
			echo '<option value="' . esc_html( $currency['name'] ) . '"' .
			selected( peachpay_get_settings_option( 'peachpay_payment_options', 'paypal_auto_convert' ), $currency['name'], true ) .
			'>' . esc_html( PEACHPAY_SUPPORTED_CURRENCIES[ $currency['name'] ] ) . '</option>';
		}
	}
	?>
	</select>
	<p class='description' for="peachpay_paypal_auto_usd">
		<?php esc_html_e( 'If a chosen currency is not supported by PayPal, switch to this currency instead of hiding PayPal. The currency switcher must be enabled for this option to work and will override country restrictions.', 'peachpay-for-woocommerce' ); ?>
	</label>
	<?php
}

/**
 * If merchnats want to enable custom intervals for currencies allow them to with this feature which will hide or show custom interval section if it's enabled.
 */
function peachpay_currency_custom_interval_cb() {
	$enabled = peachpay_get_settings_option( 'peachpay_currency_options', 'custom_intervals' );
	?>
	<input type="checkbox"
	name="peachpay_currency_options[custom_intervals]"
	id="enable_peachpay_currency_custom_intervals"
	value="1"
	<?php checked( 1, peachpay_get_settings_option( 'peachpay_currency_options', 'custom_intervals' ), true ); ?>
	>
	<label for="enable_peachpay_currency_custom_intervals">
	<?php esc_html_e( 'Enable custom intervals to be selected for currencies', 'peachpay-for-woocommerce' ); ?>
	</label>
	<?php
}
