<?php
/**
 * PeachPay shortcode implementation.
 *
 * @package PeachPay
 */

if ( ! defined( 'PEACHPAY_ABSPATH' ) ) {
	exit;
}

/**
 * PeachPay Shortcode.
 *
 * @param array $atts Shortcode Attributes.
 */
function peachpay_shortcode( $atts ) {
	$attributes = shortcode_atts(
		array(
			'product_id' => null,
		),
		$atts
	);

	if ( is_null( $attributes['product_id'] ) ) {
		return;
	}

	$product = wc_get_product( (int) $attributes['product_id'] );

	if ( is_null( $product ) || ! $product ) {
		return;
	}

	wp_localize_script(
		'pp-button-shortcode',
		'peachpayShortcodeData',
		array( 'product_id' => (int) $attributes['product_id'] )
	);

	$button_text      = peachpay_get_settings_option( 'peachpay_button_options', 'peachpay_button_text', peachpay_get_translated_text( 'button_text' ) );
	$width            = 'width:' . peachpay_get_settings_option( 'peachpay_button_options', 'button_width_product_page', '220' ) . 'px;';
	$background_color = '--pp-button-background-color:' . peachpay_get_settings_option( 'peachpay_button_options', 'button_color', PEACHPAY_DEFAULT_BACKGROUND_COLOR ) . ';';
	$text_color       = '--pp-button-text-color:' . peachpay_get_settings_option( 'peachpay_button_options', 'button_text_color', PEACHPAY_DEFAULT_TEXT_COLOR ) . ';';
	$border_radius    = 'border-radius: ' . peachpay_get_settings_option( 'peachpay_button_options', 'button_border_radius', 5 ) . 'px;';
	$style            = $width . $background_color . $text_color . $border_radius;
	$default_fonts    = peachpay_get_settings_option( 'peachpay_button_options', 'disable_default_font_css', false );
	$button_effect    = peachpay_get_settings_option( 'peachpay_button_options', 'button_effect', 'none' );
	$alignment        = peachpay_get_settings_option( 'peachpay_button_options', 'product_button_alignment', 'left' );
	$alignment_style  = 'text-align:' . $alignment . ' !important; justify-content: ' . $alignment . ' !important;';
	$icon             = peachpay_get_settings_option( 'peachpay_button_options', 'button_icon', 'none' );
	$icon_url         = get_icon_url( $icon );
	$display_cards    = peachpay_get_settings_option( 'peachpay_button_options', 'button_display_payment_method_icons', false );

	$output = '<div id="pp-button-container-shortcode" class="button-container pp-button-container " style="' . ( esc_html( $alignment ) !== 'full' ? esc_html( $alignment_style ) : '' ) . '">
		<div id="pp-stripe-payment-request-btn" style="width: 100%; margin: 5px 0; display: none;">
		</div>
		<button id="pp-button-shortcode" class="pp-button pp-product-page ' . ( $default_fonts ? '' : 'pp-button-default-font' ) . ' ' . ( esc_html( $button_effect ) === 'fade' ? 'pp-effect-fade' : '' ) . '" type="button" style="display: block; ' . esc_html( $style ) . 'font-size: 16px;">
			<object type="image/svg+xml" data="' . esc_html( peachpay_url( 'public/img/spinner.svg' ) ) . '" id="loading-spinner" class="pp-spinner hide" height="20" width="20">
			</object>
			<div id="pp-button-content">
				<span id="pp-button-text">' . esc_html( $button_text ) . '</span>
				<img id="button-icon-regular" class="pp-btn-symbol ' . ( esc_html( $icon ) === 'none' ? 'hide' : '' ) . '" src="' . esc_html( $icon_url ) . '">
			</div>
		</button>
		<div id="payment-methods-container" class="cc-company-logos ' . ( $display_cards ? '' : 'hide' ) . '">
			<img class="cc-logo" src="' . esc_html( peachpay_url( 'public/img/marks/paypal.svg' ) ) . '">
			<img class="cc-logo" src="' . esc_html( peachpay_url( 'public/img/marks/affirm.svg' ) ) . '">
			<img class="cc-logo" src="' . esc_html( peachpay_url( 'public/img/marks/klarna.svg' ) ) . '">
			<img class="cc-logo" src="' . esc_html( peachpay_url( 'public/img/marks/afterpay.svg' ) ) . '">
			<img class="cc-logo" src="' . esc_html( peachpay_url( 'public/img/marks/visa.svg' ) ) . '">
			<img class="cc-logo" src="' . esc_html( peachpay_url( 'public/img/marks/amex.svg' ) ) . '">
			<img class="cc-logo" src="' . esc_html( peachpay_url( 'public/img/marks/mastercard.svg' ) ) . '">
		</div>
	</div>';

	return $output;
}

/**
 * Gets the icon URL based on the selected icon.
 *
 * @param string $icon Selected icon.
 */
function get_icon_url( $icon ) {
	switch ( $icon ) {
		case 'lock':
			return peachpay_url( 'public/img/lock.svg' );
		case 'baseball':
			return peachpay_url( 'public/img/baseball-ball-solid.svg' );
		case 'arrow':
			return peachpay_url( 'public/img/chevron-circle-right-solid.svg' );
		case 'mountain':
			return peachpay_url( 'public/img/mountain-black.svg' );
		case 'bag':
			return peachpay_url( 'public/img/briefcase-solid.svg' );
		case 'shopping_cart':
			return peachpay_url( 'public/img/shopping-cart-solid.svg' );
		case 'none':
		default:
			return '';
	}
}
